from .compliance_form import (
    ComplianceFormButtonConfig,
    ComplianceFormSignatureButtonConfig,
)
from .compliance_task import (
    ComplianceTaskButtonConfig,
    ReviewComplianceTaskButtonConfig,
)
from .risk_managment import *



from guardian.core import ObjectPermissionChecker
from rest_framework.permissions import IsAuthenticated
from wbcore.permissions.shortcuts import is_internal_user

from wbcompliance.models.risk_management import RiskRule


class RulePermission(IsAuthenticated):
    def has_permission(self, request, view):
        permission = super().has_permission(request, view)
        if rule_id := view.kwargs.get("rule_id", None):
            checker = ObjectPermissionChecker(request.user)
            rule = RiskRule.objects.get(id=rule_id)
            permission &= checker.has_perm(rule.view_perm_str, rule)
        return permission

    def has_object_permission(self, request, view, obj):
        permission = super().has_object_permission(request, view, obj)
        # Handle case when checked_objectincidentRelationship object is given
        if incident := getattr(obj, "incident", None):
            obj = incident
        if rule := getattr(obj, "rule", None):
            checker = ObjectPermissionChecker(request.user)
            permission &= checker.has_perm(rule.view_perm_str, rule)
        return permission


def is_admin_compliance(request):
    return is_internal_user(request.user) and request.user.has_perm("wbcompliance.administrate_compliance")


def is_internal_employee_compliance(request):
    return is_internal_user(request.user) and not is_admin_compliance(request)



from django.utils.translation import gettext as _
from wbcore.menus import ItemPermission, MenuItem

from wbcompliance.permissions import is_admin_compliance

COMPLIANCETYPE_MENUITEM = MenuItem(
    label=_("Administrators"),
    endpoint="wbcompliance:compliancetype-list",
    permission=ItemPermission(
        method=is_admin_compliance,
        permissions=["wbcompliance.view_compliancetype", "wbcompliance.administrate_compliance"],
    ),
    add=MenuItem(
        label=_("Add Compliance Type"),
        endpoint="wbcompliance:compliancetype-list",
        permission=ItemPermission(method=is_admin_compliance, permissions=["wbcompliance.administrate_compliance"]),
    ),
)



from contextlib import suppress
from types import DynamicClassAttribute
from typing import Self

from celery import shared_task
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.template import Context, Template
from django.utils.translation import gettext_lazy as _
from wbcore.contrib.color.enums import WBColor
from wbcore.contrib.icons import WBIcon
from wbcore.models import WBModel
from wbcore.utils.models import ComplexToStringMixin

from .incidents import CheckedObjectIncidentRelationship


class RiskCheck(ComplexToStringMixin, WBModel):
    class CheckStatus(models.TextChoices):
        PENDING = "PENDING", "Pending"
        RUNNING = "RUNNING", "Running"
        FAILED = "FAILED", "Failed"
        SUCCESS = "SUCCESS", "Success"
        WARNING = "WARNING", "Warning"

        @DynamicClassAttribute
        def icon(self):
            return {
                "PENDING": WBIcon.PENDING.icon,
                "RUNNING": WBIcon.RUNNING.icon,
                "FAILED": WBIcon.REJECT.icon,
                "SUCCESS": WBIcon.CONFIRM.icon,
                "WARNING": WBIcon.WARNING.icon,
            }[self.value]

        @DynamicClassAttribute
        def color(self):
            return {
                "PENDING": WBColor.YELLOW_LIGHT.value,
                "RUNNING": WBColor.BLUE_LIGHT.value,
                "FAILED": WBColor.RED_LIGHT.value,
                "SUCCESS": WBColor.GREEN_LIGHT.value,
                "WARNING": WBColor.YELLOW_DARK.value,
            }[self.value]

    rule = models.ForeignKey(to="wbcompliance.RiskRule", related_name="checks", on_delete=models.CASCADE)

    creation_datetime = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_("Creation Date"),
        help_text=_("Time at which the check was created/triggered"),
    )
    evaluation_date = models.DateField(
        verbose_name=_("Evaluation Date"), help_text=_("The date at which the rule was evaluated")
    )

    checked_object_content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, related_name="triggered_checks"
    )
    checked_object_id = models.PositiveIntegerField()
    checked_object = GenericForeignKey("checked_object_content_type", "checked_object_id")
    checked_object_repr = models.CharField(max_length=256, blank=True, null=True)

    status = models.CharField(
        max_length=32, default=CheckStatus.PENDING, choices=CheckStatus.choices, verbose_name=_("Status")
    )

    @property
    def needs_incident(self) -> bool:
        # if the there is a similar checked object relationship, we needs to create a general incident if the check evaluates to a breach
        return self.rule.checked_object_relationships.filter(
            checked_object_content_type=self.checked_object_content_type, checked_object_id=self.checked_object_id
        ).exists()

    @property
    def previous_check(self) -> Self | None:
        """
        Property holding the last valid check

        Returns:
            The last valid check
        """
        with suppress(RiskCheck.DoesNotExist):
            return (
                RiskCheck.objects.filter(
                    evaluation_date__lt=self.evaluation_date,
                    rule=self.rule,
                    checked_object_content_type=self.checked_object_content_type,
                    checked_object_id=self.checked_object_id,
                )
                .order_by("-evaluation_date", "-creation_datetime")
                .first()
            )

    def save(self, *args, **kwargs):
        self.checked_object_repr = str(self.checked_object)
        super().save(*args, **kwargs)

    def compute_str(self) -> str:
        return _("{} - {}").format(
            self.checked_object,
            self.evaluation_date,
        )

    def evaluate(
        self, *explicit_dto, override_incident: bool = False, ignore_informational_threshold: bool = False
    ) -> list[models.Model]:
        """
        Evaluate the check and returns tuple of incidents information
        Args:
            override_incident: True if the existing incident needs to be overriden

        Returns:

        """
        self.status = self.CheckStatus.RUNNING
        self.save()
        rule_backend = self.rule.rule_backend.backend(
            self.evaluation_date, self.checked_object, self.rule.parameters, self.rule.thresholds.all()
        )
        self.status = self.CheckStatus.SUCCESS
        incidents = []
        report_template = Template(self.rule.rule_backend.incident_report_template)
        for incident_result in rule_backend.check_rule(*explicit_dto):
            if (
                ignore_informational_threshold
                and incident_result.severity.is_ignorable
                and incident_result.severity.is_informational
            ):
                self.status = self.CheckStatus.WARNING
            else:
                self.status = self.CheckStatus.FAILED
                # If the check is passive, we aggregated incident per breached object and return it for further processing
            if self.needs_incident:
                report = report_template.render(Context({"report_details": incident_result.report_details}))
                incident, created = self.rule.get_or_create_incident(
                    self.evaluation_date,
                    incident_result.severity,
                    incident_result.breached_object,
                    incident_result.breached_object_repr,
                )
                incident.update_or_create_relationship(
                    self,
                    report,
                    incident_result.report_details,
                    incident_result.breached_value,
                    incident_result.severity,
                    override_incident=override_incident or created,
                )
                incidents.append(incident)
            else:
                # If the check is active, the only thing that matter is whether the check led to incident or not
                report = f"<p><strong>{incident_result.breached_object_repr}</strong>: {incident_result.breached_value}</p>"  # as we do not store the breached object, we need a way to display it in the subincident relationship. This is not optimal though
                CheckedObjectIncidentRelationship.objects.create(
                    rule_check=self,
                    report=report,
                    report_details=incident_result.report_details,
                    breached_value=incident_result.breached_value,
                    severity=incident_result.severity,
                )
        self.save()
        return incidents

    class Meta:
        verbose_name = "Risk Check"
        verbose_name_plural = "Risk Checks"

    @classmethod
    def get_endpoint_basename(cls) -> str:
        return "wbcompliance:riskcheck"

    @classmethod
    def get_representation_endpoint(cls) -> str:
        return "wbcompliance:riskcheckrepresentation-list"

    @classmethod
    def get_representation_value_key(cls) -> str:
        return "id"


@shared_task
def evaluate_as_task(
    check_id: int, *dto, override_incident: bool = False, ignore_informational_threshold: bool = False
):
    check = RiskCheck.objects.get(id=check_id)
    check.evaluate(
        *dto, override_incident=override_incident, ignore_informational_threshold=ignore_informational_threshold
    )



from .compliance import (
    ComplianceActionFactory,
    ComplianceEventFactory,
    ComplianceFormFactory,
    ComplianceFormRuleFactory,
    ComplianceFormSectionFactory,
    ComplianceFormSignatureFactory,
    ComplianceFormSignatureRuleFactory,
    ComplianceFormSignatureSectionFactory,
    ComplianceFormTypeFactory,
    ComplianceTaskFactory,
    ComplianceTaskGroupFactory,
    ComplianceTaskInstanceFactory,
    ComplianceTaskInstanceReviewFactory,
    ComplianceTaskInstanceReviewNoGroupFactory,
    ComplianceTaskReviewFactory,
    ComplianceTaskReviewNoGroupFactory,
    ComplianceTypeFactory,
    UnsignedComplianceFormSignatureFactory,
)
from .risk_management import *



import pytest

from wbcompliance.viewsets import ComplianceFormSignatureModelViewSet

from .mixins import UserTestMixin


@pytest.mark.django_db
class TestSpecificFilters(UserTestMixin):
    def test_filter_version(self, compliance_form_factory, user):
        compliance_form = compliance_form_factory(assigned_to=(user.groups.first(),))
        compliance_form.activationrequested()
        compliance_form.active()
        compliance_form.draft()
        compliance_form.activationrequested()
        compliance_form.active()
        mvs = ComplianceFormSignatureModelViewSet()
        qs = mvs.get_serializer_class().Meta.model.objects.all()
        assert mvs.filterset_class().filter_version(qs, "", False) == qs
        assert mvs.filterset_class().filter_version(qs, "", False).count() == 2
        assert mvs.filterset_class().filter_version(qs, "", True).count() == 1

    def test_not_created_m2m_field_compliance_form_factory(self, compliance_form_factory):
        assert compliance_form_factory.build()



from .backend import AbstractRuleBackend
from .checks import RiskCheck
from .incidents import CheckedObjectIncidentRelationship, RiskIncident, RiskIncidentType
from .rules import RiskRule, RuleBackend, RuleCheckedObjectRelationship, RuleThreshold



from wbcompliance.permissions import (
    is_admin_compliance,
    is_internal_employee_compliance,
)

from .compliance_form import COMPLIANCEFORM_MENU, COMPLIANCEFORMSIGNATURE_MENUITEM
from .compliance_task import (
    COMPLIANCE_SETTINGS_MENU,
    COMPLIANCEEVENT_MENUITEM,
    COMPLIANCEREPORT_MENU,
)
from .compliance_type import COMPLIANCETYPE_MENUITEM
from .risk_management import (
    RISK_INCIDENT_MENUITEM,
    RISKINCIDENTTABLE_MENUITEM,
    RULE_MENUITEM,
)



from datetime import datetime, timedelta

import pandas as pd
from django.contrib.contenttypes.models import ContentType
from django.db.models import F, Q
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext as _
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response
from reversion.views import RevisionMixin
from wbcore import viewsets
from wbcore.contrib.authentication.authentication import JWTCookieAuthentication
from wbcore.pandas import fields as pf
from wbcore.pandas.views import PandasAPIViewSet

from wbcompliance.filters import (
    ComplianceActionFilter,
    ComplianceEventFilter,
    ComplianceTaskFilter,
    ComplianceTaskGroupFilter,
    ComplianceTaskInstanceFilter,
    ComplianceTaskMatrixFilter,
    ComplianceTaskReviewFilter,
    ReviewComplianceTaskFilter,
)
from wbcompliance.models import (
    ComplianceAction,
    ComplianceEvent,
    ComplianceTask,
    ComplianceTaskGroup,
    ComplianceTaskInstance,
    ReviewComplianceTask,
    update_or_create_compliance_document,
)
from wbcompliance.serializers import (
    ComplianceActionModelSerializer,
    ComplianceEventModelSerializer,
    ComplianceTaskGroupModelSerializer,
    ComplianceTaskGroupRepresentationSerializer,
    ComplianceTaskInstanceListModelSerializer,
    ComplianceTaskInstanceModelSerializer,
    ComplianceTaskModelSerializer,
    ComplianceTaskRepresentationSerializer,
    ComplianceTaskReviewModelSerializer,
    ReviewComplianceTaskModelSerializer,
    ReviewComplianceTaskRepresentationSerializer,
)

from .buttons import ComplianceTaskButtonConfig, ReviewComplianceTaskButtonConfig
from .display import (
    ComplianceActionDisplayConfig,
    ComplianceEventDisplayConfig,
    ComplianceTaskDisplayConfig,
    ComplianceTaskGroupDisplayConfig,
    ComplianceTaskInstanceComplianceTaskDisplayConfig,
    ComplianceTaskInstanceDisplayConfig,
    ComplianceTaskInstanceReviewDisplayConfig,
    ComplianceTaskMatrixPandasDisplayConfig,
    ComplianceTaskReviewDisplayConfig,
    ReviewComplianceTaskDisplayConfig,
)
from .endpoints import (
    ComplianceActionEndpointConfig,
    ComplianceEventEndpointConfig,
    ComplianceTaskComplianceTaskGroupEndpointConfig,
    ComplianceTaskEndpointConfig,
    ComplianceTaskGroupEndpointConfig,
    ComplianceTaskInstanceComplianceTaskEndpointConfig,
    ComplianceTaskInstanceEndpointConfig,
    ComplianceTaskInstanceReviewGroupEndpointConfig,
    ComplianceTaskInstanceReviewNoGroupEndpointConfig,
    ComplianceTaskMatrixEndpointConfig,
    ComplianceTaskReviewGroupEndpointConfig,
    ComplianceTaskReviewNoGroupEndpointConfig,
    ReviewComplianceTaskEndpointConfig,
)
from .titles import (
    ComplianceTaskComplianceTaskGroupTitleConfig,
    ComplianceTaskGroupTitleConfig,
    ComplianceTaskInstanceTitleConfig,
    ComplianceTaskMatrixPandasTitleConfig,
    ComplianceTaskTitleConfig,
    ReviewComplianceTaskTitleConfig,
)


class ComplianceTaskGroupRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbcompliance:compliancetaskgrouprepresentation"
    search_fields = ["name"]
    ordering_fields = ["name"]

    queryset = ComplianceTaskGroup.objects.all()
    serializer_class = ComplianceTaskGroupRepresentationSerializer


class ComplianceTaskRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbcompliance:compliancetaskrepresentation"
    search_fields = ["title"]
    ordering_fields = ["title", "occurrence", "active", "type"]

    queryset = ComplianceTask.objects.all()
    serializer_class = ComplianceTaskRepresentationSerializer


class ReviewComplianceTaskRepresentationViewSet(viewsets.RepresentationViewSet):
    IDENTIFIER = "wbcompliance:reviewcompliancetaskrepresentation"
    search_fields = ordering_fields = ["title"]
    ordering = ["-year", "-occured", "title"]
    filterset_class = ReviewComplianceTaskFilter

    queryset = ReviewComplianceTask.objects.all()
    serializer_class = ReviewComplianceTaskRepresentationSerializer


class ComplianceTaskGroupModelViewSet(RevisionMixin, viewsets.ModelViewSet):
    IDENTIFIER = "wbcompliance:compliancetaskgroup"
    display_config_class = ComplianceTaskGroupDisplayConfig
    endpoint_config_class = ComplianceTaskGroupEndpointConfig
    title_config_class = ComplianceTaskGroupTitleConfig

    search_fields = ["name"]
    ordering_fields = ["order", "name"]
    ordering = ["order", "name"]

    filterset_class = ComplianceTaskGroupFilter

    serializer_class = ComplianceTaskGroupModelSerializer

    queryset = ComplianceTaskGroup.objects.all()


class ComplianceTaskModelViewSet(RevisionMixin, viewsets.ModelViewSet):
    IDENTIFIER = "wbcompliance:compliancetask"
    display_config_class = ComplianceTaskDisplayConfig
    title_config_class = ComplianceTaskTitleConfig
    endpoint_config_class = ComplianceTaskEndpointConfig
    button_config_class = ComplianceTaskButtonConfig

    search_fields = ["title"]
    ordering_fields = ["title", "occurrence", "risk_level", "group", "active", "type", "review"]

    filterset_class = ComplianceTaskFilter

    serializer_class = ComplianceTaskModelSerializer

    queryset = ComplianceTask.objects.select_related(
        "group",
        "type",
    ).prefetch_related("review")

    @action(detail=True, methods=["PATCH"], authentication_classes=[JWTCookieAuthentication])
    def generate_instance(self, request, pk):
        if not request.user.is_superuser:
            return Response(status=status.HTTP_403_FORBIDDEN)

        if occured := request.POST.get("occured", None):
            task = get_object_or_404(ComplianceTask, pk=pk)
            task.generate_compliance_task_instance(occured=datetime.strptime(occured, "%Y-%m-%d"))
        else:
            return Response(status=status.HTTP_400_BAD_REQUEST)

        return Response(
            {"__notification": {"title": "Indicator Instance has been generated."}},
            status=status.HTTP_200_OK,
        )


class ComplianceTaskComplianceTaskGroupModelViewSet(ComplianceTaskModelViewSet):
    title_config_class = ComplianceTaskComplianceTaskGroupTitleConfig
    endpoint_config_class = ComplianceTaskComplianceTaskGroupEndpointConfig

    def get_queryset(self):
        return super().get_queryset().filter(group__id=self.kwargs["group_id"])


class ComplianceTaskInstanceModelViewSet(RevisionMixin, viewsets.ModelViewSet):
    IDENTIFIER = "wbcompliance:compliancetaskinstance"
    display_config_class = ComplianceTaskInstanceDisplayConfig
    endpoint_config_class = ComplianceTaskInstanceEndpointConfig
    title_config_class = ComplianceTaskInstanceTitleConfig

    search_fields = ["task__title"]
    ordering_fields = ["task__title", "occured", "status", "review"]
    ordering = ["-occured", "-type_name", "group_name", "task__title"]

    filterset_class = ComplianceTaskInstanceFilter

    serializer_class = ComplianceTaskInstanceModelSerializer

    queryset = ComplianceTaskInstance.objects.select_related("task").prefetch_related("review")

    def get_serializer_class(self):
        if self.get_action() in ["list", "list-metadata"]:
            return ComplianceTaskInstanceListModelSerializer
        return ComplianceTaskInstanceModelSerializer

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .annotate(
                task_title=F("task__title"),
                active=F("task__active"),
                type_name=F("task__type__name"),
                group_name=F("task__group__name"),
            )
        )


class ComplianceTaskInstanceComplianceTaskModelViewSet(ComplianceTaskInstanceModelViewSet):
    IDENTIFIER = "wbcompliance:compliancetask-compliancetaskinstance"
    display_config_class = ComplianceTaskInstanceComplianceTaskDisplayConfig
    endpoint_config_class = ComplianceTaskInstanceComplianceTaskEndpointConfig

    def get_queryset(self):
        if task_id := self.kwargs.get("task_id", None):
            return (
                super()
                .get_queryset()
                .filter(task__id=task_id)
                .annotate(
                    task_title=F("task__title"),
                    active=F("task__active"),
                    type_name=F("task__type__name"),
                    group_name=F("task__group__name"),
                )
            )
        return ComplianceTask.objects.none()


class ComplianceTaskMatrixPandasViewSet(PandasAPIViewSet):
    IDENTIFIER = "wbcommission:compliancetaskmatrix"
    queryset = ComplianceTaskInstance.objects.all()

    display_config_class = ComplianceTaskMatrixPandasDisplayConfig
    title_config_class = ComplianceTaskMatrixPandasTitleConfig
    endpoint_config_class = ComplianceTaskMatrixEndpointConfig

    filterset_class = ComplianceTaskMatrixFilter

    def get_pandas_fields(self, request):
        _fields = [
            pf.PKField("id", label=_("ID")),
            pf.CharField(key="type_name", label=_("Administrator")),
            pf.CharField(key="group_name", label=_("Group")),
            pf.CharField(key="task_title", label=_("Tasks")),
        ]
        if _dict := ComplianceTaskInstance.get_dict_max_count_task():
            qs_occured = (
                ComplianceTaskInstance.objects.filter(task=_dict.get("task"))
                .order_by("-occured")
                .values_list("occured", flat=True)
                .distinct()[:12]
            )
            for date in qs_occured:
                date_str = date.strftime("%Y-%m-%d")
                covered_month = (date - timedelta(days=1)).strftime("%Y-%h")
                _fields.append(pf.CharField(key=date_str, label=covered_month))
        return pf.PandasFields(fields=tuple(_fields))

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .filter(task__active=True)
            .annotate(task_title=F("task__title"), type_name=F("task__type__name"), group_name=F("task__group__name"))
        )

    def get_dataframe(self, request, queryset, **kwargs):
        def _rename_columns(df):
            pass

            rename_map = {}
            for col in df.columns:
                if not isinstance(col, str):
                    rename_map[col] = str(col)
            rename_map["index"] = "task"
            return df.rename(columns=rename_map)

        df = pd.DataFrame()
        if queryset.exists():
            df = pd.DataFrame(
                queryset.values("task", "type_name", "group_name", "task_title", "occured", "status"),
                columns=["task", "type_name", "group_name", "task_title", "occured", "status"],
            )
            df["status"] = df["status"].apply(lambda x: ComplianceTaskInstance.Status[x].label)
            df = df.where(pd.notnull(df), "")
            df = df.pivot_table(
                index=["task", "type_name", "group_name", "task_title"],
                columns="occured",
                values="status",
                aggfunc="last",
            )
            df = _rename_columns(df)
            df = df.reset_index()
            df["id"] = df.index

        return df


class ComplianceActionModelViewSet(RevisionMixin, viewsets.ModelViewSet):
    IDENTIFIER = "wbcompliance:complianceaction"
    display_config_class = ComplianceActionDisplayConfig
    endpoint_config_class = ComplianceActionEndpointConfig

    search_fields = ["title"]
    ordering_fields = [
        "title",
        "deadline",
        "progress",
        "status",
        "active",
        "type",
        "creator",
        "changer",
        "last_modified",
    ]
    ordering = ["-last_modified", "active"]

    filterset_class = ComplianceActionFilter

    serializer_class = ComplianceActionModelSerializer

    queryset = ComplianceAction.objects.select_related("type", "creator", "changer")

    def get_queryset(self):
        if self.request.user.has_perm("wbcompliance.administrate_compliance"):
            return super().get_queryset()
        return ComplianceAction.objects.none()


class TypeComplianceActionModelViewSet(ComplianceActionModelViewSet):
    IDENTIFIER = "wbcompliance:type-complianceaction"

    def get_queryset(self):
        return super().get_queryset().filter(type=self.kwargs["type_id"])


class ComplianceEventModelViewSet(RevisionMixin, viewsets.ModelViewSet):
    IDENTIFIER = "wbcompliance:complianceevent"
    display_config_class = ComplianceEventDisplayConfig
    endpoint_config_class = ComplianceEventEndpointConfig

    search_fields = ["title"]
    ordering_fields = ["title", "type", "level", "active", "type", "creator", "last_modified", "confidential"]
    ordering = ["-last_modified", "active"]

    filterset_class = ComplianceEventFilter

    serializer_class = ComplianceEventModelSerializer

    queryset = ComplianceEvent.objects.select_related(
        "type",
        "creator",
        "changer",
    )

    def get_queryset(self):
        qs = super().get_queryset()
        if self.request.user.has_perm("wbcompliance.administrate_compliance"):
            return qs
        return qs.filter(Q(creator=self.request.user.profile) | Q(confidential=False))


class TypeComplianceEventModelViewSet(ComplianceEventModelViewSet):
    IDENTIFIER = "wbcompliance:type-complianceevent"

    def get_queryset(self):
        return super().get_queryset().filter(type=self.kwargs["type_id"])


class ReviewComplianceTaskModelViewSet(RevisionMixin, viewsets.ModelViewSet):
    IDENTIFIER = "wbcompliance:reviewcompliancetask"
    display_config_class = ReviewComplianceTaskDisplayConfig
    endpoint_config_class = ReviewComplianceTaskEndpointConfig
    button_config_class = ReviewComplianceTaskButtonConfig
    title_config_class = ReviewComplianceTaskTitleConfig

    search_fields = ["title"]
    ordering = ["-year", "-is_instance", "-occured", "-changed", "title"]
    ordering_fields = [
        "title",
        "from_date",
        "to_date",
        "year",
        "status",
        "changer",
        "changed",
        "is_instance",
        "occured",
        "review_task",
        "ocurrance",
    ]

    filterset_class = ReviewComplianceTaskFilter

    serializer_class = ReviewComplianceTaskModelSerializer

    queryset = ReviewComplianceTask.objects.select_related(
        "creator",
        "changer",
        "review_task",
        "type",
    )

    @action(detail=True, methods=["PATCH"], authentication_classes=[JWTCookieAuthentication])
    def regenerate_document(self, request, pk):
        if not request.user.has_perm("wbcompliance.administrate_compliance"):
            return Response({}, status=status.HTTP_403_FORBIDDEN)
        content_type = ContentType.objects.get_for_model(ReviewComplianceTask)
        send_email = request.POST.get("send_email", "false") == "true"
        update_or_create_compliance_document.delay(request.user.id, content_type.id, pk, send_email=send_email)

        return Response(
            {"__notification": {"title": "PDF is going to be created and sent to you."}},
            status=status.HTTP_200_OK,
        )

    @action(detail=True, methods=["PATCH"], authentication_classes=[JWTCookieAuthentication])
    def generate_instance(self, request, pk):
        if not request.user.has_perm("wbcompliance.administrate_compliance"):
            return Response({}, status=status.HTTP_403_FORBIDDEN)
        review = ReviewComplianceTask.objects.get(id=pk)
        review.generate_review_compliance_task_instance(link_instance=True)
        return Response(
            {"__notification": {"title": "Instance is going to be generate."}},
            status=status.HTTP_200_OK,
        )

    @action(detail=True, methods=["PATCH"], authentication_classes=[JWTCookieAuthentication])
    def link_tasks(self, request, pk):
        if not request.user.has_perm("wbcompliance.administrate_compliance"):
            return Response({}, status=status.HTTP_403_FORBIDDEN)

        if (_type := request.POST.get("type", None)) and (operation := request.POST.get("operation", None)):
            review = ReviewComplianceTask.objects.get(id=pk)
            for task in ComplianceTask.objects.filter(type__name=_type):
                if operation == "ADD":
                    task.review.add(review)
                elif operation == "REMOVE":
                    task.review.remove(review)
                task.save()
        else:
            return Response({}, status=status.HTTP_400_BAD_REQUEST)

        return Response(
            {"__notification": {"title": "Tasks of a type is going to be linked."}},
            status=status.HTTP_200_OK,
        )


class ComplianceTaskReviewNoGroupModelViewSet(ComplianceTaskModelViewSet):
    display_config_class = ComplianceTaskReviewDisplayConfig
    endpoint_config_class = ComplianceTaskReviewNoGroupEndpointConfig
    filterset_class = ComplianceTaskReviewFilter
    serializer_class = ComplianceTaskReviewModelSerializer

    def get_queryset(self):
        return super().get_queryset().filter(Q(review__id=self.kwargs["review_id"]) & Q(group=None))


class ComplianceTaskReviewGroupModelViewSet(ComplianceTaskModelViewSet):
    display_config_class = ComplianceTaskReviewDisplayConfig
    endpoint_config_class = ComplianceTaskReviewGroupEndpointConfig
    filterset_class = ComplianceTaskReviewFilter
    serializer_class = ComplianceTaskReviewModelSerializer

    def get_queryset(self):
        return super().get_queryset().filter(Q(review__id=self.kwargs["review_id"]) & Q(group=self.kwargs["group_id"]))


class ComplianceTaskInstanceReviewNoGroupModelViewSet(ComplianceTaskInstanceModelViewSet):
    IDENTIFIER = "wbcompliance:review-compliancetaskinstancenogroup"
    display_config_class = ComplianceTaskInstanceReviewDisplayConfig
    endpoint_config_class = ComplianceTaskInstanceReviewNoGroupEndpointConfig

    def get_serializer_class(self):
        return ComplianceTaskInstanceModelSerializer

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .filter(Q(review__id=self.kwargs["review_id"]) & Q(task__group=None))
            .annotate(
                task_title=F("task__title"),
                active=F("task__active"),
                type_name=F("task__type__name"),
                group_name=F("task__group__name"),
            )
        )


class ComplianceTaskInstanceReviewGroupModelViewSet(ComplianceTaskInstanceModelViewSet):
    IDENTIFIER = "wbcompliance:review-compliancetaskinstancegroup"
    display_config_class = ComplianceTaskInstanceReviewDisplayConfig
    endpoint_config_class = ComplianceTaskInstanceReviewGroupEndpointConfig

    def get_serializer_class(self):
        return ComplianceTaskInstanceModelSerializer

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .filter(Q(review__id=self.kwargs["review_id"]) & Q(task__group=self.kwargs["group_id"]))
            .annotate(
                task_title=F("task__title"),
                active=F("task__active"),
                type_name=F("task__type__name"),
                group_name=F("task__group__name"),
            )
        )



from datetime import timedelta

from django.utils.translation import gettext as _
from psycopg.types.range import NumericRange
from rest_framework import serializers
from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.content_type.serializers import ContentTypeRepresentationSerializer
from wbcore.contrib.authentication.serializers import GroupRepresentationSerializer
from wbcore.contrib.directory.serializers import PersonRepresentationSerializer

from wbcompliance.models.risk_management import (
    CheckedObjectIncidentRelationship,
    RiskIncident,
    RiskIncidentType,
    RuleThreshold,
)

from .checks import RiskCheckRepresentationSerializer
from .rules import RiskRuleRepresentationSerializer


class RiskIncidentTypeRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = RiskIncidentType
        fields = ("id", "name")


class RiskIncidentRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = RiskIncident
        fields = ("id", "computed_str")


class CheckedObjectIncidentRelationshipRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = CheckedObjectIncidentRelationship
        fields = ("id", "computed_str")


class RuleThresholdModelSerializer(wb_serializers.ModelSerializer):
    _rule = RiskRuleRepresentationSerializer(source="rule")
    _notifiable_users = PersonRepresentationSerializer(source="notifiable_users", many=True)
    _notifiable_groups = GroupRepresentationSerializer(source="notifiable_groups", many=True)
    _severity = RiskIncidentTypeRepresentationSerializer(source="severity")
    range_lower = wb_serializers.FloatField(source="range.lower", allow_null=True, required=False, precision=4)
    range_upper = wb_serializers.FloatField(source="range.upper", allow_null=True, required=False, precision=4)
    range = wb_serializers.DecimalRangeField(required=False)

    def validate(self, data):
        range_dict = data.pop("range", {})
        range_upper = range_dict.pop("upper", self.instance.range.upper if self.instance else None)
        range_lower = range_dict.pop("lower", self.instance.range.lower if self.instance else None)
        if range_upper and range_lower and range_upper < range_lower:
            raise serializers.ValidationError({"range": "Lower needs to be strictly lower than upper bound"})
        data["range"] = NumericRange(lower=range_lower, upper=range_upper)
        return data

    class Meta:
        model = RuleThreshold
        read_only_fields = ("computed_str",)
        fields = (
            "id",
            "rule",
            "_rule",
            "range_lower",
            "range_upper",
            "range",
            "severity",
            "notifiable_users",
            "_notifiable_users",
            "notifiable_groups",
            "_notifiable_groups",
            "computed_str",
            "upgradable_after_days",
            "_severity",
            "severity",
        )


class RiskIncidentModelSerializer(wb_serializers.ModelSerializer):
    # Extra fields to accommodate the multi level tree view with the CheckedIncidentRelationship class
    status_icon = wb_serializers.IconSelectField(read_only=True)
    _group_key = wb_serializers.CharField(read_only=True)
    checked_date = wb_serializers.DateField(read_only=True)
    object_repr = wb_serializers.CharField(read_only=True)
    threshold_repr = wb_serializers.CharField(read_only=True, required=False)
    breached_value = wb_serializers.TextField(read_only=True, default="Open to see details")
    report = wb_serializers.TextField(read_only=True, default="Open to see details")

    _resolved_by = PersonRepresentationSerializer(source="resolved_by")
    _breached_content_type = ContentTypeRepresentationSerializer(source="breached_content_type")
    _rule = RiskRuleRepresentationSerializer(source="rule")
    _severity = RiskIncidentTypeRepresentationSerializer(source="severity")
    date_range = wb_serializers.DateRangeField(outward_bounds_transform="[]")
    ignore_until = wb_serializers.DateField(
        read_only=True, label="Ignore Until (Included)", help_text=_("Ignore until this date (included)")
    )
    ignore_duration_in_days = wb_serializers.IntegerField(
        required=False,
        label=_("Ignore for X days"),
        help_text=_(
            "If set to a value different than 0, will ignore the forthcoming incidents for the specified number of days"
        ),
    )

    @wb_serializers.register_resource()
    def additional_resources(self, instance, request, user):
        res = {}
        if instance.checked_object_relationships.exists():
            res["relationships"] = reverse(
                "wbcompliance:riskincident-relationship-list",
                args=[instance.id],
                request=request,
            )
        return res

    def validate(self, data):
        if (ignore_duration_in_days := data.get("ignore_duration_in_days", None)) is not None:
            data["ignore_duration"] = timedelta(days=ignore_duration_in_days)
        return data

    class Meta:
        model = RiskIncident
        only_fsm_transition_on_instance = True

        fields = (
            "id",
            "status_icon",
            "date_range",
            "last_ignored_date",
            "ignore_until",
            "rule",
            "_rule",
            "breached_content_type",
            "_breached_content_type",
            "breached_object_id",
            "breached_object_repr",
            "ignore_duration",
            "ignore_duration_in_days",
            "status",
            "severity",
            "comment",
            "resolved_by",
            "_resolved_by",
            "_severity",
            "severity",
            "is_notified",
            "_additional_resources",
            "_group_key",
            "checked_date",
            "object_repr",
            "threshold_repr",
            "breached_value",
            "report",
        )
        read_only_fields = (
            "id",
            "date_range",
            "last_ignored_date",
            "ignore_until",
            "rule",
            "breached_content_type",
            "breached_object_id",
            "breached_object_repr",
            "status",
            "severity",
            "resolved_by",
            "_severity",
            "severity",
            "is_notified",
        )


class CheckedObjectIncidentRelationshipModelSerializer(wb_serializers.ModelSerializer):
    _resolved_by = PersonRepresentationSerializer(source="resolved_by")
    _incident = RiskIncidentRepresentationSerializer(source="incident")
    _rule_check = RiskCheckRepresentationSerializer(source="rule_check")
    _severity = RiskIncidentTypeRepresentationSerializer(source="severity")
    breached_value = wb_serializers.TextField()
    # extra annotation to play properly with the tree table
    checked_date = wb_serializers.DateField(read_only=True, required=False)
    # rule = wb_serializers.PrimaryKeyRelatedField()
    # _rule = RiskRuleRepresentationSerializer(source="rule")
    object_repr = wb_serializers.CharField(read_only=True, required=False)
    date_range = wb_serializers.DateRangeField(read_only=True, outward_bounds_transform="[]", required=False)
    threshold_repr = wb_serializers.CharField(read_only=True, required=False)

    class Meta:
        model = CheckedObjectIncidentRelationship
        read_only_fields = (
            "computed_str",
            "incident",
            "status",
            "severity",
            "rule_check",
            "report",
            "_severity",
            "resolved_by",
            "checked_date",
            "rule",
            "_rule",
            "object_repr",
            "date_range",
            "breached_value",
        )
        fields = (
            "id",
            "computed_str",
            "rule_check",
            "_rule_check",
            "incident",
            "_incident",
            "report",
            "status",
            "comment",
            "resolved_by",
            "_resolved_by",
            "_severity",
            "severity",
            "_additional_resources",
            "checked_date",
            # "rule",
            # "_rule",
            "object_repr",
            "date_range",
            "threshold_repr",
            "breached_value",
        )



from django.utils.translation import gettext as _
from wbcore.metadata.configs.titles import TitleViewConfig

from wbcompliance.models import ComplianceTaskGroup


class ComplianceTaskMatrixPandasTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Indicator Matrix")


class ComplianceTaskComplianceTaskGroupTitleConfig(TitleViewConfig):
    def get_list_title(self):
        if group_id := self.view.kwargs.get("group_id"):
            group = ComplianceTaskGroup.objects.get(id=group_id)
            return _("Indicators of Group: {}").format(group.name)
        return self.get_list_title()


class ComplianceTaskTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Indicator: {{title}}")

    def get_list_title(self):
        return _("Indicators")

    def get_create_title(self):
        return _("New Indicator")


class ComplianceTaskGroupTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Group Indicator: {{name}}")

    def get_list_title(self):
        return _("Groups Indicator")

    def get_create_title(self):
        return _("New Group Indicator")


class ComplianceTaskInstanceTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Instance Indicator: {{_task.title}} -  {{occured}}")

    def get_list_title(self):
        return _("Instances Indicator")

    def get_create_title(self):
        return _("New Instance Indicator")


class ReviewComplianceTaskTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Report Indicator: {{computed_str}}")

    def get_list_title(self):
        return _("Reports Indicator")

    def get_create_title(self):
        return _("New Report Indicator")



from .compliance_form import (
    ComplianceFormSectionRuleTitleConfig,
    ComplianceFormSectionTitleConfig,
    ComplianceFormSignatureSectionRuleTitleConfig,
    ComplianceFormSignatureTitleConfig,
    ComplianceFormTitleConfig,
)
from .compliance_task import (
    ComplianceTaskComplianceTaskGroupTitleConfig,
    ComplianceTaskGroupTitleConfig,
    ComplianceTaskInstanceTitleConfig,
    ComplianceTaskMatrixPandasTitleConfig,
    ComplianceTaskTitleConfig,
    ReviewComplianceTaskTitleConfig,
)
from .compliance_type import ComplianceTypeTitleConfig
from .risk_managment import *



from typing import Optional

from django.utils.translation import gettext as _
from wbcore.contrib.color.enums import WBColor
from wbcore.enums import Unit
from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.view_config import DisplayViewConfig

from .incidents import get_formatting, get_legends


class RiskManagementIncidentTableDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        if content_type := self.view.checked_object_content_type:
            field_name = content_type.name
        else:
            field_name = "Checked object"
        fields = [
            dp.Field(
                key="checked_object_repr",
                label=field_name,
                width=Unit.PIXEL(250),
                formatting_rules=[dp.FormattingRule(style={"fontWeight": "bold"})],
            )
        ]
        for key, label in self.view.get_rule_map:
            fields.append(
                dp.Field(
                    key=key,
                    label=label,
                    width=Unit.PIXEL(200),
                    formatting_rules=[
                        dp.FormattingRule(
                            condition=("==", -1),
                            style={"backgroundColor": WBColor.GREEN_LIGHT.value, "color": WBColor.GREEN_LIGHT.value},
                        ),
                        *get_formatting(id_field="severity_order", color=True).formatting_rules,
                    ],
                )
            )
        return dp.ListDisplay(
            fields=fields,
            legends=[
                dp.Legend(
                    items=[
                        dp.LegendItem(icon=WBColor.GREEN_LIGHT.value, label=_("No Incident")),
                        *get_legends().items,
                    ]
                )
            ],
        )



from django.utils.translation import gettext as _
from wbcore import serializers as wb_serializers
from wbcore.contrib.icons import WBIcon
from wbcore.enums import RequestType
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.buttons.view_config import ButtonViewConfig
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    create_simple_display,
)


class RiskRuleButtonConfig(ButtonViewConfig):
    def get_custom_instance_buttons(self):
        class CheckSerializer(wb_serializers.Serializer):
            start = wb_serializers.DateField(label=_("Start"))
            end = wb_serializers.DateField(label=_("End"))

        return {
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("risk_management:riskcheck",),
                key="recheck",
                label=_("Recheck Rules"),
                icon=WBIcon.REGENERATE.icon,
                description_fields=_(
                    """
                <p>Recheck all the rules between a certain interval</p>
                """
                ),
                serializer=CheckSerializer,
                action_label=_("recheck"),
                title=_("Recheck Rules"),
                instance_display=create_simple_display([["start"], ["end"]]),
            )
        }



from .compliance_form import (
    CFComplianceFormSignatureModelViewSet,
    ComplianceFormModelViewSet,
    ComplianceFormRepresentationViewSet,
    ComplianceFormRuleViewSet,
    ComplianceFormSectionComplianceFormViewSet,
    ComplianceFormSectionRepresentationViewSet,
    ComplianceFormSectionRuleViewSet,
    ComplianceFormSectionViewSet,
    ComplianceFormSignatureModelViewSet,
    ComplianceFormSignatureSectionRepresentationViewSet,
    ComplianceFormSignatureSectionRuleViewSet,
    ComplianceFormTypeRepresentationViewSet,
    ComplianceFormTypeViewSet,
)
from .compliance_task import (
    ComplianceActionModelViewSet,
    ComplianceEventModelViewSet,
    ComplianceTaskComplianceTaskGroupModelViewSet,
    ComplianceTaskGroupModelViewSet,
    ComplianceTaskGroupRepresentationViewSet,
    ComplianceTaskInstanceComplianceTaskModelViewSet,
    ComplianceTaskInstanceModelViewSet,
    ComplianceTaskInstanceReviewGroupModelViewSet,
    ComplianceTaskInstanceReviewNoGroupModelViewSet,
    ComplianceTaskMatrixPandasViewSet,
    ComplianceTaskModelViewSet,
    ComplianceTaskRepresentationViewSet,
    ComplianceTaskReviewGroupModelViewSet,
    ComplianceTaskReviewNoGroupModelViewSet,
    ReviewComplianceTaskModelViewSet,
    ReviewComplianceTaskRepresentationViewSet,
    TypeComplianceActionModelViewSet,
    TypeComplianceEventModelViewSet,
)
from .compliance_type import (
    ComplianceTypeModelViewSet,
    ComplianceTypeRepresentationViewSet,
)
from .risk_management import *



import pandas as pd
from django.contrib.auth import get_user_model
from django.db.models import Case, CharField, Count, OuterRef, Q, Subquery, Value, When
from django.db.models.functions import Coalesce
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from rest_framework import status
from rest_framework.decorators import action
from wbcore import serializers as wb_serializers
from wbcore import viewsets
from wbcore.contrib.guardian.viewsets.mixins import GuardianFilterMixin
from wbcore.utils.date import get_date_interval_from_request

from wbcompliance.filters import RiskRuleFilterSet
from wbcompliance.models.risk_management import (
    RiskIncident,
    RiskRule,
    RuleBackend,
    RuleCheckedObjectRelationship,
    RuleThreshold,
)
from wbcompliance.models.risk_management.rules import RuleGroup, process_rule_as_task
from wbcompliance.permissions import RulePermission
from wbcompliance.serializers import (
    RiskRuleModelSerializer,
    RiskRuleRepresentationSerializer,
    RuleBackendRepresentationSerializer,
    RuleCheckedObjectRelationshipModelSerializer,
    RuleCheckedObjectRelationshipRepresentationSerializer,
    RuleGroupRepresentationSerializer,
    RuleThresholdModelSerializer,
    RuleThresholdRepresentationSerializer,
)
from wbcompliance.viewsets.buttons import RiskRuleButtonConfig
from wbcompliance.viewsets.display import (
    RiskRuleDisplayConfig,
    RuleCheckedObjectRelationshipRiskRuleDisplayConfig,
    RuleThresholdRiskRuleDisplayConfig,
)
from wbcompliance.viewsets.endpoints import (
    RiskRuleEndpointConfig,
    RuleCheckedObjectRelationshipRiskRuleEndpointConfig,
    RuleThresholdRiskRuleEndpointConfig,
)

User = get_user_model()


class RuleGroupRepresentationViewSet(viewsets.RepresentationViewSet):
    queryset = RuleGroup.objects.all()
    serializer_class = RuleGroupRepresentationSerializer
    search_fields = ("name",)


class RuleCheckedObjectRelationshipRepresentationViewSet(viewsets.RepresentationViewSet):
    queryset = RuleCheckedObjectRelationship.objects.all()
    serializer_class = RuleCheckedObjectRelationshipRepresentationSerializer
    search_fields = ("checked_object_repr",)


class RuleBackendRepresentationViewSet(viewsets.RepresentationViewSet):
    queryset = RuleBackend.objects.all()
    serializer_class = RuleBackendRepresentationSerializer
    search_fields = ("name",)


class RuleThresholdRepresentationViewSet(viewsets.RepresentationViewSet):
    queryset = RuleThreshold.objects.all()
    serializer_class = RuleThresholdRepresentationSerializer


class RiskRuleRepresentationViewSet(GuardianFilterMixin, viewsets.RepresentationViewSet):
    queryset = RiskRule.objects.all()
    serializer_class = RiskRuleRepresentationSerializer
    search_fields = (
        "name",
        "description",
        "rule_backend__name",
    )


class RuleThresholdRiskRuleModelViewSet(viewsets.ModelViewSet):
    permission_classes = [RulePermission]
    display_config_class = RuleThresholdRiskRuleDisplayConfig
    endpoint_config_class = RuleThresholdRiskRuleEndpointConfig
    queryset = RuleThreshold.objects.all()
    serializer_class = RuleThresholdModelSerializer
    filterset_fields = {
        "severity": ["exact"],
    }

    def get_queryset(self):
        return super().get_queryset().filter(rule_id=self.kwargs["rule_id"])


class RuleCheckedObjectRelationshipRiskRuleModelViewSet(viewsets.ModelViewSet):
    permission_classes = [RulePermission]
    display_config_class = RuleCheckedObjectRelationshipRiskRuleDisplayConfig
    endpoint_config_class = RuleCheckedObjectRelationshipRiskRuleEndpointConfig
    queryset = RuleCheckedObjectRelationship.objects.all()
    serializer_class = RuleCheckedObjectRelationshipModelSerializer
    search_fields = ["checked_object_repr"]
    filterset_fields = {
        "checked_object_repr": ["icontains"],
    }

    def get_queryset(self):
        return super().get_queryset().filter(rule_id=self.kwargs["rule_id"])


class RiskRuleModelViewSet(GuardianFilterMixin, viewsets.ModelViewSet):
    display_config_class = RiskRuleDisplayConfig
    endpoint_config_class = RiskRuleEndpointConfig
    button_config_class = RiskRuleButtonConfig

    queryset = RiskRule.objects.all()
    serializer_class = RiskRuleModelSerializer
    filterset_class = RiskRuleFilterSet

    search_fields = ("name", "description", "checked_object_relationships__checked_object_repr")
    ordering_fields = (
        "name",
        "is_enable",
        "only_passive_check_allowed",
        "is_silent",
        "is_mandatory",
        "open_incidents_count",
    )
    ordering = ("name",)

    def get_serializer_class(self):
        if "pk" in self.kwargs:
            if rule := self.get_object():
                parameter_fields = rule.rule_backend.backend_class.get_serializer_class().get_parameter_fields()

                class TmpSerializer(RiskRuleModelSerializer):
                    class Meta(RiskRuleModelSerializer.Meta):
                        flatten_fields = {
                            "parameters": wb_serializers.JSONTableField(
                                serializer_class=rule.rule_backend.backend_class.get_serializer_class(),
                                flatten_field_names=parameter_fields,
                            )
                        }

                return TmpSerializer
        return RiskRuleModelSerializer

    def _subquery_open_risk_incidents_count(self):
        queryset = RiskIncident.objects.filter(status=RiskIncident.Status.OPEN, rule=OuterRef("pk"))
        return Coalesce(Subquery(queryset.values("rule").annotate(c=Count("rule")).values("c")[:1]), 0)

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .annotate(
                open_incidents_count=self._subquery_open_risk_incidents_count(),
                in_breach=Case(
                    When(Q(open_incidents_count__gt=0) & Q(is_enable=True), then=Value("BREACH")),
                    When(Q(open_incidents_count=0) & Q(is_enable=True), then=Value("PASSED")),
                    default=Value("INACTIVE"),
                    output_field=CharField(),
                ),
            )
            .select_related(
                "rule_backend",
                "creator",
            )
            .prefetch_related("incidents")
        )

    @action(detail=True, methods=["PATCH"])
    def recheck(self, request, pk=None):
        start, end = get_date_interval_from_request(request, request_type="POST")
        if start and end:
            rule = get_object_or_404(RiskRule, pk=pk)
            for evaluation_date in pd.date_range(start, end, freq="B"):
                process_rule_as_task.delay(rule.id, evaluation_date, override_incident=False)
            return HttpResponse("Rule is checking", status=status.HTTP_200_OK)
        return HttpResponse("Wrong arguments", status=status.HTTP_400_BAD_REQUEST)



from django.utils.translation import gettext as _
from wbcore.menus import ItemPermission, Menu, MenuItem

from wbcompliance.permissions import is_admin_compliance

COMPLIANCEFORM_MENUITEM = MenuItem(
    label=_("Forms"),
    endpoint="wbcompliance:complianceform-list",
    permission=ItemPermission(
        method=is_admin_compliance,
        permissions=["wbcompliance.view_complianceform", "wbcompliance.administrate_compliance"],
    ),
    add=MenuItem(
        label=_("Add Form"),
        endpoint="wbcompliance:complianceform-list",
        permission=ItemPermission(method=is_admin_compliance, permissions=["wbcompliance.administrate_compliance"]),
    ),
)


COMPLIANCEFORMSIGNATURE_MENUITEM = MenuItem(
    label=_("Form Signatures"),
    endpoint="wbcompliance:complianceformsignature-list",
    permission=ItemPermission(permissions=["wbcompliance.view_complianceformsignature"]),
)

ADMIN_COMPLIANCEFORMSIGNATURE_MENUITEM = MenuItem(
    label=_("Form Signatures"),
    endpoint="wbcompliance:complianceformsignature-list",
    permission=ItemPermission(method=is_admin_compliance, permissions=["wbcompliance.view_complianceformsignature"]),
)

COMPLIANCEFORMTYPE_MENUITEM = MenuItem(
    label=_("Form Types"),
    endpoint="wbcompliance:complianceformtype-list",
    permission=ItemPermission(
        method=is_admin_compliance,
        permissions=["wbcompliance.view_complianceformtype", "wbcompliance.administrate_compliance"],
    ),
    add=MenuItem(
        label=_("Add Form Type"),
        endpoint="wbcompliance:complianceformtype-list",
        permission=ItemPermission(method=is_admin_compliance, permissions=["wbcompliance.administrate_compliance"]),
    ),
)

COMPLIANCEFORM_MENU = Menu(
    label=_("Forms"),
    items=[COMPLIANCEFORM_MENUITEM, ADMIN_COMPLIANCEFORMSIGNATURE_MENUITEM, COMPLIANCEFORMTYPE_MENUITEM],
)



# Generated by Django 4.2.9 on 2024-02-20 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcompliance", "0011_alter_riskrule_parameters"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="compliancetype",
            options={
                "permissions": [("administrate_compliance", "Can Administrate Compliance")],
                "verbose_name": "Compliance Type",
                "verbose_name_plural": "Compliance Types",
            },
        ),
    ]



from django.utils.translation import gettext as _
from wbcore.metadata.configs.titles import TitleViewConfig

from wbcompliance.models import (
    ComplianceForm,
    ComplianceFormRule,
    ComplianceFormSection,
    ComplianceFormSignatureRule,
    ComplianceFormSignatureSection,
)


class ComplianceFormTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Compliance Form: {{title}} - version {{version}}")

    def get_list_title(self):
        return _("Compliance Forms")

    def get_create_title(self):
        return _("New Compliance Form")


class ComplianceFormSignatureTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        return _("Compliance Form Signature: {{_compliance_form.title}} - version {{_compliance_form.version}}")

    def get_list_title(self):
        return _("Compliance Form Signatures")

    def get_create_title(self):
        return _("New Compliance Form Signature")


class ComplianceFormSectionTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        if ref_id := self.view.kwargs.get("pk"):
            _section = ComplianceFormSection.objects.get(id=ref_id)
            return _("Compliance Form Section: {} of Compliance Form: - version {}").format(
                _section.name, _section.compliance_form.title, _section.compliance_form.version
            )

        return _("Compliance Form Section: {{name}}")

    def get_list_title(self):
        if compliance_form_id := self.view.kwargs.get("compliance_form_id"):
            compliance_form = ComplianceForm.objects.get(id=compliance_form_id)
            return _("Compliance Form: {} - version {}").format(compliance_form.title, compliance_form.version)
        return _("Compliance Form Sections")

    def get_create_title(self):
        if compliance_form_id := self.view.kwargs.get("compliance_form_id"):
            compliance_form = ComplianceForm.objects.get(id=compliance_form_id)
            return _("New Section of Compliance Form: {} - version {}").format(
                compliance_form.title, compliance_form.version
            )
        return _("New Section")


class ComplianceFormSectionRuleTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        if ref_id := self.view.kwargs.get("pk"):
            _rule = ComplianceFormRule.objects.get(id=ref_id)
            return _("Compliance Form Rule: {} of Section: {}").format(_rule.id, _rule.section.name)
        return _("Compliance Form Rule: {{id}}")

    def get_list_title(self):
        if section_id := self.view.kwargs.get("section_id"):
            section = ComplianceFormSection.objects.get(id=section_id)
            return _("Compliance Form: {} - version {}").format(
                section.compliance_form.title, section.compliance_form.version
            )
        return _("Compliance Form Rules")

    def get_create_title(self):
        if section_id := self.view.kwargs.get("section_id"):
            section = ComplianceFormSection.objects.get(id=section_id)
            return _("New Rule of Section: {}").format(section.name)
        return _("New Rule")


class ComplianceFormSignatureSectionRuleTitleConfig(TitleViewConfig):
    def get_instance_title(self):
        if ref_id := self.view.kwargs.get("pk"):
            _rule = ComplianceFormSignatureRule.objects.get(id=ref_id)
            return _("Compliance Form Signature Rule: {} of Section: {}").format(_rule.id, _rule.section.name)
        return _("Compliance Form Signature Rule: {{id}}")

    def get_list_title(self):
        if section_id := self.view.kwargs.get("section_id"):
            section = ComplianceFormSignatureSection.objects.get(id=section_id)
            return _("Compliance Form Signature: {} - version {}").format(
                section.compliance_form_signature.compliance_form.title,
                section.compliance_form_signature.compliance_form.version,
            )
        return _("Compliance Form Signature Rules")

    def get_create_title(self):
        if section_id := self.view.kwargs.get("section_id"):
            section = ComplianceFormSignatureSection.objects.get(id=section_id)
            return _("New Rule of Section: {}").format(section.name)
        return _("New Rule")



from django.apps import AppConfig
from django.db.models.signals import post_migrate

from wbcompliance.management import autodiscover_riskmanagement_backends


class WbcomplianceConfig(AppConfig):
    name = "wbcompliance"

    #
    def ready(self) -> None:
        post_migrate.connect(
            autodiscover_riskmanagement_backends,
            dispatch_uid="wbcrm.synchronization.initialize_task",
        )



from datetime import timedelta

import factory
from faker import Faker
from psycopg.types.range import DateRange

from wbcompliance.models.risk_management import (
    CheckedObjectIncidentRelationship,
    RiskIncident,
    RiskIncidentType,
)

from .checks import RiskCheckFactory
from .rules import RuleCheckedObjectRelationshipFactory

fake = Faker()


def _generate_date_range():
    d1 = fake.date_object()
    return DateRange(lower=d1, upper=d1 + timedelta(days=fake.pyint(min_value=1, max_value=100)))


def _extract_date_range_from_incident(incident):
    return fake.date_between_dates(incident.date_range.lower, incident.date_range.upper)


class RiskIncidentMixinFactory:
    status = RiskIncident.Status.OPEN.value
    comment = factory.Faker("paragraph")
    resolved_by = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")


def get_severity_order(o):
    types = RiskIncidentType.objects.all()
    if types.exists():
        return types.last().severity_order + 1
    return 0


def _get_severity(o):
    if o.rule.thresholds.exists():
        return o.rule.thresholds.first().severity
    return RiskIncidentTypeFactory.create()


class RiskIncidentTypeFactory(factory.django.DjangoModelFactory):
    name = "LOW"
    severity_order = factory.LazyAttribute(lambda o: get_severity_order(o))
    is_ignorable = True
    is_automatically_closed = False
    is_informational = False

    class Meta:
        model = RiskIncidentType


class RiskIncidentFactory(RiskIncidentMixinFactory, factory.django.DjangoModelFactory):
    rule = factory.SubFactory("wbcompliance.factories.risk_management.rules.RiskRuleFactory")
    severity = factory.LazyAttribute(lambda x: x.rule.thresholds.first().severity)
    date_range = factory.LazyAttribute(lambda o: _generate_date_range())

    breached_content_type = None
    breached_object_id = None
    breached_object_repr = factory.Faker("text", max_nb_chars=32)

    class Meta:
        model = RiskIncident


def _create_risk_check(incident):
    rel = RuleCheckedObjectRelationshipFactory.create(rule=incident.rule)
    return RiskCheckFactory.create(
        rule=rel.rule,
        checked_object_content_type=rel.checked_object_content_type,
        checked_object_id=rel.checked_object_id,
    )


class CheckedObjectIncidentRelationshipFactory(RiskIncidentMixinFactory, factory.django.DjangoModelFactory):
    severity = factory.LazyAttribute(lambda x: x.incident.rule.thresholds.first().severity)
    rule_check = factory.LazyAttribute(lambda x: _create_risk_check(x.incident))
    incident = factory.SubFactory("wbcompliance.factories.risk_management.incidents.RiskIncidentFactory")
    # rule_check = factory.SubFactory("wbcompliance.factories.risk_management.checks.RiskCheckFactory")
    report = factory.Faker("paragraph")
    report_details = factory.LazyAttribute(lambda x: dict(a="b"))

    class Meta:
        model = CheckedObjectIncidentRelationship



from django.contrib.auth import get_user_model
from django.contrib.postgres.aggregates import StringAgg
from django.db import models
from django.db.models import F
from wbcore import viewsets

from wbcompliance.filters.risk_management.checks import RiskCheckFilterSet
from wbcompliance.models.risk_management import RiskCheck
from wbcompliance.serializers import (
    RiskCheckModelSerializer,
    RiskCheckRepresentationSerializer,
)
from wbcompliance.viewsets.buttons import RiskCheckButtonConfig
from wbcompliance.viewsets.display import RiskCheckDisplayConfig
from wbcompliance.viewsets.endpoints import RiskCheckEndpointConfig

User = get_user_model()


class RiskCheckRepresentationViewSet(viewsets.RepresentationViewSet):
    queryset = RiskCheck.objects.all()
    serializer_class = RiskCheckRepresentationSerializer


class RiskCheckModelViewSet(viewsets.ModelViewSet):
    button_config_class = RiskCheckButtonConfig
    display_config_class = RiskCheckDisplayConfig
    endpoint_config_class = RiskCheckEndpointConfig
    queryset = RiskCheck.objects.all()
    serializer_class = RiskCheckModelSerializer

    filterset_class = RiskCheckFilterSet

    def get_queryset(self):
        return (
            RiskCheck.objects.annotate(
                status_icon=models.Case(
                    *[
                        models.When(status=value, then=models.Value(RiskCheck.CheckStatus[value].icon))
                        for value in RiskCheck.CheckStatus.values
                    ],
                    default=models.Value("none"),
                ),
                incident_reports=StringAgg(F("incidents__report"), delimiter="<br>"),
            )
            .select_related(
                "rule",
                "checked_object_content_type",
            )
            .prefetch_related("incidents")
        )



from contextlib import suppress

from django.core.exceptions import MultipleObjectsReturned
from django.db.utils import ProgrammingError

from .rules import RuleBackend, RuleGroup


def register(backend_name: str | None, incident_report_template: str | None = None, rule_group_key: str | None = None):
    """
    Decorator to include when a backend need automatic registration
    Args:
        backend_name:

    Returns:

    """
    if not backend_name:
        raise ValueError("At least one name must be passed to register.")

    def _decorator(backend_class):
        with suppress(RuntimeError, MultipleObjectsReturned, ProgrammingError):
            defaults = {
                "name": backend_name,
                "allowed_checked_object_content_type": backend_class.get_allowed_content_type(),
            }
            if incident_report_template:
                defaults["incident_report_template"] = incident_report_template
            if rule_group_key:
                defaults["rule_group"] = RuleGroup.objects.get_or_create(
                    key=rule_group_key, defaults={"name": rule_group_key.title()}
                )[0]

            RuleBackend.objects.update_or_create(
                backend_class_path=backend_class.__module__,
                backend_class_name=backend_class.__name__,
                defaults=defaults,
            )
        return backend_class

    return _decorator



import factory
import pytz

from wbcompliance.models import (
    ComplianceAction,
    ComplianceEvent,
    ComplianceForm,
    ComplianceFormRule,
    ComplianceFormSection,
    ComplianceFormSignature,
    ComplianceFormSignatureRule,
    ComplianceFormSignatureSection,
    ComplianceFormType,
    ComplianceTask,
    ComplianceTaskGroup,
    ComplianceTaskInstance,
    ComplianceType,
    ReviewComplianceTask,
)


class ComplianceFormTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceFormType

    name = factory.Faker("text", max_nb_chars=32)


class ComplianceFormFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceForm

    form_type = factory.SubFactory(ComplianceFormTypeFactory)
    creator = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")
    created = factory.Faker("date_time", tzinfo=pytz.utc)
    changer = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")
    changed = factory.Faker("date_time", tzinfo=pytz.utc)
    title = factory.Faker("text", max_nb_chars=64)
    policy = factory.Faker("paragraph", nb_sentences=5)
    only_internal = factory.Faker("pybool")
    start = factory.Faker("date_between", start_date="+2d", end_date="+3d")
    end = factory.Faker("date_between", start_date="+4d", end_date="+5d")
    compliance_type = factory.SubFactory("wbcompliance.factories.ComplianceTypeFactory")

    @factory.post_generation
    def assigned_to(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for group in extracted:
                self.assigned_to.add(group)


class UnsignedComplianceFormSignatureFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceFormSignature

    compliance_form = factory.SubFactory(ComplianceFormFactory)
    policy = factory.Faker("paragraph", nb_sentences=5)
    person = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")
    remark = factory.Faker("paragraph", nb_sentences=5)
    start = factory.Faker("date_between", start_date="+2d", end_date="+3d")
    end = factory.Faker("date_between", start_date="+4d", end_date="+5d")


class ComplianceFormSignatureFactory(UnsignedComplianceFormSignatureFactory):
    signed = factory.Faker("date_time", tzinfo=pytz.utc)


class ComplianceFormSectionFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceFormSection

    compliance_form = factory.SubFactory(ComplianceFormFactory)
    name = factory.Faker("text", max_nb_chars=32)


class ComplianceFormRuleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceFormRule

    section = factory.SubFactory(ComplianceFormSectionFactory)
    text = factory.Faker("text", max_nb_chars=255)


class ComplianceFormSignatureSectionFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceFormSignatureSection

    compliance_form_signature = factory.SubFactory(UnsignedComplianceFormSignatureFactory)
    name = factory.Faker("text", max_nb_chars=32)


class ComplianceFormSignatureRuleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceFormSignatureRule

    section = factory.SubFactory(ComplianceFormSignatureSectionFactory)
    text = factory.Faker("text", max_nb_chars=255)


class ComplianceTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceType

    name = factory.Faker("text", max_nb_chars=32)
    description = factory.Faker("text", max_nb_chars=255)

    @factory.post_generation
    def in_charge(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for group in extracted:
                self.in_charge.add(group)


class ComplianceTaskGroupFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceTaskGroup

    name = factory.Faker("text", max_nb_chars=32)
    order = factory.Faker("pyint", min_value=0, max_value=9999)


class ParentReviewComplianceTaskFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ReviewComplianceTask

    title = factory.Faker("text", max_nb_chars=32)
    from_date = factory.Faker("date_between", start_date="+2d", end_date="+3d")
    to_date = factory.Faker("date_between", start_date="+4d", end_date="+5d")
    description = factory.Faker("text", max_nb_chars=255)
    year = factory.Faker("year")
    creator = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")
    created = factory.Faker("date_time", tzinfo=pytz.utc)
    changer = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")
    changed = factory.Faker("date_time", tzinfo=pytz.utc)
    review_task = None
    occured = factory.Faker("date")
    type = factory.SubFactory("wbcompliance.factories.ComplianceTypeFactory")


class ReviewComplianceTaskFactory(ParentReviewComplianceTaskFactory):
    class Meta:
        model = ReviewComplianceTask

    review_task = factory.SubFactory(ParentReviewComplianceTaskFactory)


class ComplianceTaskFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceTask

    title = factory.Faker("text", max_nb_chars=32)
    description = factory.Faker("text", max_nb_chars=255)
    group = factory.SubFactory(ComplianceTaskGroupFactory)
    type = factory.SubFactory(ComplianceTypeFactory)
    risk_level = "LOW"
    remarks = factory.Faker("text", max_nb_chars=255)

    @factory.post_generation
    def review(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for item in extracted:
                self.review.add(item)


class ComplianceTaskReviewFactory(ComplianceTaskFactory):
    @factory.post_generation
    def review(self, create, extracted, **kwargs):
        review = ReviewComplianceTaskFactory()
        self.review.add(review)


class ComplianceTaskReviewNoGroupFactory(ComplianceTaskReviewFactory):
    group = None


class ComplianceTaskInstanceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceTaskInstance

    text = factory.Faker("text", max_nb_chars=255)
    summary_text = factory.Faker("text", max_nb_chars=255)
    task = factory.SubFactory(ComplianceTaskFactory)

    @factory.post_generation
    def review(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for item in extracted:
                self.review.add(item)


class ComplianceTaskInstanceReviewFactory(ComplianceTaskInstanceFactory):
    @factory.post_generation
    def review(self, create, extracted, **kwargs):
        review = ReviewComplianceTaskFactory()
        self.review.add(review)


class ComplianceTaskInstanceReviewNoGroupFactory(ComplianceTaskInstanceFactory):
    @factory.post_generation
    def review(self, create, extracted, **kwargs):
        self.task.group = None
        review = ReviewComplianceTaskFactory()
        self.review.add(review)
        self.task.save()


class ComplianceActionFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceAction

    title = factory.Faker("text", max_nb_chars=32)
    description = factory.Faker("text", max_nb_chars=255)
    summary_description = factory.Faker("text", max_nb_chars=255)
    deadline = factory.Faker("date_between", start_date="+2d", end_date="+3d")
    type = factory.SubFactory(ComplianceTypeFactory)
    creator = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")
    created = factory.Faker("date_time", tzinfo=pytz.utc)
    changer = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")


class ComplianceEventFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ComplianceEvent

    title = factory.Faker("text", max_nb_chars=32)
    exec_summary = factory.Faker("text", max_nb_chars=255)
    exec_summary_board = factory.Faker("text", max_nb_chars=255)
    description = factory.Faker("text", max_nb_chars=255)
    actions_taken = factory.Faker("text", max_nb_chars=255)
    consequences = factory.Faker("text", max_nb_chars=255)
    future_suggestions = factory.Faker("text", max_nb_chars=255)
    type = factory.SubFactory(ComplianceTypeFactory)
    creator = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")
    created = factory.Faker("date_time", tzinfo=pytz.utc)
    changer = factory.SubFactory("wbcore.contrib.authentication.factories.AuthenticatedPersonFactory")



from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.content_type.serializers import ContentTypeRepresentationSerializer
from wbcore.contrib.icons.serializers import IconSelectField

from wbcompliance.models.risk_management import RiskCheck
from wbcompliance.serializers.risk_management.rules import RiskRuleRepresentationSerializer


class RiskCheckRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = RiskCheck
        fields = ("id", "computed_str")


class RiskCheckModelSerializer(wb_serializers.ModelSerializer):
    _rule = RiskRuleRepresentationSerializer(source="rule")
    _checked_object_content_type = ContentTypeRepresentationSerializer(source="checked_object_content_type")

    incident_reports = wb_serializers.TextField()
    status_icon = IconSelectField(read_only=True)

    @wb_serializers.register_resource()
    def additional_resources(self, instance, request, user):
        res = {}
        if instance.incidents.exists():
            res["incidents"] = (
                f'{reverse("wbcompliance:checkedobjectincidentrelationship-list", args=[], request=request)}?rule_check={instance.id}'
            )

        return res

    class Meta:
        model = RiskCheck
        fields = (
            "id",
            "rule",
            "_rule",
            "incident_reports",
            "creation_datetime",
            "evaluation_date",
            "computed_str",
            "_checked_object_content_type",
            "checked_object_content_type",
            "checked_object_id",
            "status",
            "status_icon",
            "_additional_resources",
        )
        read_only_fields = fields



from typing import Generator

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from faker import Faker
from wbcore import serializers as wb_serializers

from wbcompliance.models.risk_management import backend

User = get_user_model()
fake = Faker()


class RuleBackend(backend.AbstractRuleBackend):
    @classmethod
    def get_serializer_class(cls):
        class RuleBackendSerializer(wb_serializers.Serializer):
            date = wb_serializers.DateField(required=False)
            name = wb_serializers.CharField(required=False)
            int = wb_serializers.IntegerField(required=False)
            anonymous_user = wb_serializers.PrimaryKeyRelatedField(
                queryset=User.objects.all(), default=User.objects.none(), required=False
            )

            @classmethod
            def get_parameter_fields(cls):
                return ["date", "name", "int", "anonymous_user"]

        return RuleBackendSerializer

    def check_rule(self, *dto_args, **kwargs) -> Generator[backend.IncidentResult, None, None]:
        severity = self.thresholds.first().severity if self.thresholds.exists() else None
        yield backend.IncidentResult(
            breached_object=None,
            breached_value=str(fake.word()),
            breached_object_repr=fake.text(max_nb_chars=125),
            report_details={"label": fake.paragraph()},
            severity=severity,
        )

    @classmethod
    def get_allowed_content_type(cls) -> "ContentType":
        return None
